/* This file is part of gacopyz.
   Copyright (C) 2006 Sergey Poznyakoff

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
   MA 02110-1301 USA */

#include <gacopyz_priv.h>

static void default__smfi_log_printer(int, char *, va_list);
static void (*__smfi_log_printer)(int, char *, va_list) = default__smfi_log_printer;

/* Logging */
static void
default__smfi_log_printer(int level, char *fmt, va_list ap)
{
	switch (level) {
	case SMI_LOG_DEBUG:
		level = LOG_DEBUG;
		break;
	case SMI_LOG_INFO:
		level = LOG_INFO;
		break;
	case SMI_LOG_WARN:
		level = LOG_WARNING;
		break;
	case SMI_LOG_ERR:
		level = LOG_ERR;
		break;

	case SMI_LOG_FATAL:
	default:
		level = LOG_EMERG;
	}
	vsyslog(level, fmt, ap); /* FIXME: if absent? */
}

void
gacopyz_logmsg(int level, char *fmt, ...)
{
	va_list ap;
	va_start(ap, fmt);
	__smfi_log_printer(level, fmt, ap);
	va_end(ap);
}

void
gacopyz_log(gacopyz_conn_t conn, int level, char *fmt, ...)
{
	if (conn->logmask & SMI_LOG_MASK(level)) {
		va_list ap;
		va_start(ap, fmt);
		__smfi_log_printer(level, fmt, ap);
		va_end(ap);
	}
}

#define VBUFSIZE 69

static size_t
format_vbuf(char vbuf[VBUFSIZE], char *buf, size_t size)
{
	char *p = vbuf;
	char *q = vbuf + 51;
	int i;
	size_t j = 0;
	
	for (i = 0; i < 16; i++) {
		unsigned c;
		if (j < size) {
			c = *(unsigned char*)buf++;
			j++;
		} else
			c = 0;
		sprintf(p, "%02X ", c);
		p += 3;
		*q++ = isprint(c) ? c : '.';
		if (i == 7) {
			*p++ = ' ';
			*q++ = ' ';
		}
	}
	*p++ = ' ';
	*p = ' ';
	*q = 0;
	return j;
}

void
gacopyz_logdump(gacopyz_conn_t conn, int level, char *pfx,
	       char *buf, size_t size)
{
	if (conn->logmask & SMI_LOG_MASK(level)) {
		char vbuf[VBUFSIZE];

		while (size) {
			size_t rd = format_vbuf(vbuf, buf, size);
			gacopyz_logmsg(level, "%s: %s", pfx, vbuf);
			size -= rd;
			buf += rd;
		}
	}
}


void
gacopyz_set_logger(void (*logger)(int, char *, va_list))
{
	__smfi_log_printer = logger;
}
